@@ -11,7 +11,7 @@ from openmdao.core.total_jac import _TotalJacInfo
 from openmdao.recorders.recording_manager import RecordingManager
 from openmdao.recorders.recording_iteration_stack import Recording
 from openmdao.utils.record_util import create_local_meta, check_path
-from openmdao.utils.general_utils import simple_warning
+from openmdao.utils.general_utils import simple_warning, warn_deprecation
 from openmdao.utils.mpi import MPI
 from openmdao.utils.options_dictionary import OptionsDictionary
 import openmdao.utils.coloring as coloring_mod
@@ -121,7 +121,13 @@ class Driver(object):
         self.recording_options = OptionsDictionary(parent_name=type(self).__name__)
 
         self.recording_options.declare('record_model_metadata', types=bool, default=True,
-                                       desc='Record metadata for all Systems in the model')
+                                       desc='Deprecated. Recording of model metadata will always '
+                                       'be done',
+                                       deprecation="The recording option, record_model_metadata, "
+                                       "on Driver is "
+                                       "deprecated. Recording of model metadata will always "
+                                       "be done",
+                                       )
         self.recording_options.declare('record_desvars', types=bool, default=True,
                                        desc='Set to True to record design variables at the '
                                             'driver level')
@@ -135,8 +141,8 @@ class Driver(object):
                                        desc='Set to True to record constraints at the '
                                             'driver level')
         self.recording_options.declare('includes', types=list, default=[],
-                                       desc='Patterns for variables to include in recording. \
-                                       Uses fnmatch wildcards')
+                                       desc='Patterns for variables to include in recording. '
+                                       'Uses fnmatch wildcards')
         self.recording_options.declare('excludes', types=list, default=[],
                                        desc='Patterns for vars to exclude in recording '
                                             '(processed post-includes). Uses fnmatch wildcards')
@@ -145,6 +151,12 @@ class Driver(object):
                                             'level')
         self.recording_options.declare('record_inputs', types=bool, default=True,
                                        desc='Set to True to record inputs at the driver level')
+        self.recording_options.declare('record_outputs', types=bool, default=True,
+                                       desc='Set True to record outputs at the '
+                                            'driver level.')
+        self.recording_options.declare('record_residuals', types=bool, default=False,
+                                       desc='Set True to record residuals at the '
+                                            'driver level.')
 
         # What the driver supports.
         self.supports = OptionsDictionary(parent_name=type(self).__name__)
@@ -153,7 +165,7 @@ class Driver(object):
         self.supports.declare('linear_constraints', types=bool, default=False)
         self.supports.declare('two_sided_constraints', types=bool, default=False)
         self.supports.declare('multiple_objectives', types=bool, default=False)
-        self.supports.declare('integer_design_vars', types=bool, default=False)
+        self.supports.declare('integer_design_vars', types=bool, default=True)
         self.supports.declare('gradients', types=bool, default=False)
         self.supports.declare('active_set', types=bool, default=False)
         self.supports.declare('simultaneous_derivatives', types=bool, default=False)
@@ -162,10 +174,7 @@ class Driver(object):
         self.iter_count = 0
         self.cite = ""
 
-        self._coloring_info = coloring_mod._DEF_COMP_SPARSITY_ARGS.copy()
-        self._coloring_info['coloring'] = None
-        self._coloring_info['dynamic'] = False
-        self._coloring_info['static'] = None
+        self._coloring_info = coloring_mod._get_coloring_meta()
 
         self._total_jac_sparsity = None
         self._res_jacs = {}
@@ -248,8 +257,8 @@ class Driver(object):
         self._total_jac = None
 
         self._has_scaling = (
-            np.any([r['scaler'] is not None for r in self._responses.values()]) or
-            np.any([dv['scaler'] is not None for dv in self._designvars.values()])
+            np.any([r['total_scaler'] is not None for r in self._responses.values()]) or
+            np.any([dv['total_scaler'] is not None for dv in self._designvars.values()])
         )
 
         # Determine if any design variables are discrete.
@@ -348,45 +357,51 @@ class Driver(object):
         # includes and excludes for outputs are specified using promoted names
         abs2prom = model._var_allprocs_abs2prom['output']
 
-        allvars = []
-        # if desvars, etc. are not wanted, we need to exclude them from the outputs even if
-        # they match the includes list.
-        skip = set()
+        # 1. If record_outputs is True, get the set of outputs
+        # 2. Filter those using includes and excludes to get the baseline set of variables to record
+        # 3. Add or remove from that set any desvars, objs, and cons based on the recording
+        #    options of those
 
+        # includes and excludes for outputs are specified using _promoted_ names
+        # vectors are keyed on absolute name, discretes on relative/promoted name
+        myinputs = myoutputs = myresiduals = []
+
+        if recording_options['record_outputs']:
+            myoutputs = sorted([n for n, prom in abs2prom.items() if check_path(prom, incl, excl)])
+
+            views = model._outputs._views
+
+            if model._var_discrete['output']:
+                # if we have discrete outputs then residual name set doesn't match output one
+                if recording_options['record_residuals']:
+                    myresiduals = [n for n in myoutputs if n in views]
+            elif recording_options['record_residuals']:
+                myresiduals = myoutputs
+
+        elif recording_options['record_residuals']:
+            myresiduals = [n for n in model._residuals._views
+                           if check_path(abs2prom[n], incl, excl)]
+
+        myoutputs = set(myoutputs)
         if recording_options['record_desvars']:
-            allvars.extend(self._designvars)
-        else:
-            skip.update(self._designvars)
+            myoutputs.update(self._designvars)
         if recording_options['record_objectives'] or recording_options['record_responses']:
-            allvars.extend(self._objs)
-        else:
-            skip.update(self._objs)
+            myoutputs.update(self._objs)
         if recording_options['record_constraints'] or recording_options['record_responses']:
-            allvars.extend(self._cons)
-        else:
-            skip.update(self._cons)
-
-        vars2record = {
-            'output': [n for n in allvars
-                       if n in abs2prom and check_path(abs2prom[n], incl, excl, True)]
-        }
+            myoutputs.update(self._cons)
 
         # inputs (if in options). inputs use _absolute_ names for includes/excludes
         if 'record_inputs' in recording_options:
             if recording_options['record_inputs']:
                 # sort the results since _var_allprocs_abs2prom isn't ordered
-                vars2record['input'] = sorted([n for n in model._var_allprocs_abs2prom['input']
-                                               if check_path(n, incl, excl)])
-            else:
-                vars2record['input'] = []
+                myinputs = sorted([n for n in model._var_allprocs_abs2prom['input']
+                                  if check_path(n, incl, excl)])
 
-        if incl:
-            # loop over abs2prom (which includes both continuous and discrete outputs) since
-            # the order doesn't matter (we're sorting it at the end).
-            vars2record['output'].extend(n for n, prom in abs2prom.items() if n not in skip and
-                                         check_path(prom, incl, excl))
-            # remove dups and make sure order is the same on all procs
-            vars2record['output'] = sorted(set(vars2record['output']))
+        vars2record = {
+            'input': myinputs,
+            'output': list(myoutputs),
+            'residual': myresiduals
+        }
 
         return vars2record
 
@@ -399,9 +414,8 @@ class Driver(object):
         self._rec_mgr.startup(self)
 
         # record the system metadata to the recorders attached to this Driver
-        if self.recording_options['record_model_metadata']:
-            for sub in self._problem().model.system_iter(recurse=True, include_self=True):
-                self._rec_mgr.record_metadata(sub)
+        for sub in self._problem().model.system_iter(recurse=True, include_self=True):
+            self._rec_mgr.record_metadata(sub)
 
     def _get_voi_val(self, name, meta, remote_vois, driver_scaling=True, rank=None):
         """
@@ -476,11 +490,11 @@ class Driver(object):
 
         if self._has_scaling and driver_scaling:
             # Scale design variable values
-            adder = meta['adder']
+            adder = meta['total_adder']
             if adder is not None:
                 val += adder
 
-            scaler = meta['scaler']
+            scaler = meta['total_scaler']
             if scaler is not None:
                 val *= scaler
 
@@ -521,7 +535,20 @@ class Driver(object):
             indices = slice(None)
 
         if name in self._designvars_discrete:
-            problem.model._discrete_outputs[name] = int(value)
+
+            # Note, drivers set values here and generally should know it is setting an integer.
+            # However, the DOEdriver may pull a non-integer value from its generator, so we
+            # convert it.
+            if isinstance(value, float):
+                value = int(value)
+            elif isinstance(value, np.ndarray):
+                if isinstance(problem.model._discrete_outputs[name], int):
+                    # Setting an integer value with a 1D array - don't want to convert to array.
+                    value = int(value)
+                else:
+                    value = value.astype(np.int)
+
+            problem.model._discrete_outputs[name] = value
 
         else:
             desvar = problem.model._outputs._views_flat[name]
@@ -529,11 +556,11 @@ class Driver(object):
 
             # Undo driver scaling when setting design var values into model.
             if self._has_scaling:
-                scaler = meta['scaler']
+                scaler = meta['total_scaler']
                 if scaler is not None:
                     desvar[indices] *= 1.0 / scaler
 
-                adder = meta['adder']
+                adder = meta['total_adder']
                 if adder is not None:
                     desvar[indices] -= adder
 
@@ -633,6 +660,8 @@ class Driver(object):
         self._objs = objs = OrderedDict()
         self._cons = cons = OrderedDict()
 
+        model._setup_driver_units()
+
         self._responses = resps = model.get_responses(recurse=True)
         for name, data in resps.items():
             if data['type'] == 'con':
@@ -666,7 +695,8 @@ class Driver(object):
         self.iter_count += 1
         return False
 
-    def _compute_totals(self, of=None, wrt=None, return_format='flat_dict', global_names=True):
+    def _compute_totals(self, of=None, wrt=None, return_format='flat_dict', global_names=None,
+                        use_abs_names=True):
         """
         Compute derivatives of desired quantities with respect to desired inputs.
 
@@ -685,7 +715,9 @@ class Driver(object):
             returns them in a dictionary whose keys are tuples of form (of, wrt). For
             the scipy optimizer, 'array' is also supported.
         global_names : bool
-            Set to True when passing in global names to skip some translation steps.
+            Deprecated.  Use 'use_abs_names' instead.
+        use_abs_names : bool
+            Set to True when passing in absolute names to skip some translation steps.
 
         Returns
         -------
@@ -702,12 +734,17 @@ class Driver(object):
             print(header)
             print(len(header) * '-' + '\n')
 
+        if global_names is not None:
+            warn_deprecation("'global_names' is deprecated in calls to _compute_totals. "
+                             "Use 'use_abs_names' instead.")
+            use_abs_names = global_names
+
         if problem.model._owns_approx_jac:
             self._recording_iter.push(('_compute_totals_approx', 0))
 
             try:
                 if total_jac is None:
-                    total_jac = _TotalJacInfo(problem, of, wrt, global_names,
+                    total_jac = _TotalJacInfo(problem, of, wrt, use_abs_names,
                                               return_format, approx=True, debug_print=debug_print)
 
                     # Don't cache linear constraint jacobian
@@ -722,7 +759,7 @@ class Driver(object):
 
         else:
             if total_jac is None:
-                total_jac = _TotalJacInfo(problem, of, wrt, global_names, return_format,
+                total_jac = _TotalJacInfo(problem, of, wrt, use_abs_names, return_format,
                                           debug_print=debug_print)
 
                 # don't cache linear constraint jacobian
@@ -841,11 +878,16 @@ class Driver(object):
             raise RuntimeError("Driver '%s' does not support simultaneous derivatives." %
                                self._get_name())
 
-    def _setup_tot_jac_sparsity(self):
+    def _setup_tot_jac_sparsity(self, coloring=None):
         """
         Set up total jacobian subjac sparsity.
 
         Drivers that can use subjac sparsity should override this.
+
+        Parameters
+        ----------
+        coloring : Coloring or None
+            Current coloring.
         """
         pass
 
@@ -1031,15 +1073,30 @@ def record_iteration(requester, prob, case_name):
     filt = requester._filtered_vars_to_record
     model = prob.model
 
+    inputs, outputs, residuals = model.get_nonlinear_vectors()
+
     parallel = requester._rec_mgr._check_parallel() if model.comm.size > 1 else False
 
-    outs = model._retrieve_data_of_kind(filt, 'output', 'nonlinear', parallel)
-    ins = model._retrieve_data_of_kind(filt, 'input', 'nonlinear', parallel)
+    discrete_inputs = model._discrete_inputs
+    discrete_outputs = model._discrete_outputs
 
-    data = {
-        'output': outs,
-        'input': ins
-    }
+    data = {'input': {}, 'output': {}, 'residual': {}}
+    if requester.recording_options['record_inputs'] and (inputs._names or len(discrete_inputs) > 0):
+        data['input'] = model._retrieve_data_of_kind(filt, 'input', 'nonlinear', parallel)
+
+    if requester.recording_options['record_outputs'] and \
+            (outputs._names or len(discrete_outputs) > 0):
+        data['output'] = model._retrieve_data_of_kind(filt, 'output', 'nonlinear', parallel)
+
+    if requester.recording_options['record_residuals'] and residuals._names:
+        data['residual'] = model._retrieve_data_of_kind(filt, 'residual', 'nonlinear', parallel)
+
+    from openmdao.core.problem import Problem
+    if isinstance(requester, Problem):
+        if requester.recording_options['record_derivatives'] and \
+                prob.driver._designvars and prob.driver._responses:
+            totals = requester.compute_totals(return_format='flat_dict_structured_key')
+            data['totals'] = totals
 
     requester._rec_mgr.record_iteration(requester, data,
                                         requester._get_recorder_metadata(case_name))
